// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"encoding/json"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"google.golang.org/grpc/status"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"smart-enterprise-management/utils"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"github.com/jaryhe/gopkgs/tasker/transtasker"
)

func checkProjectUpdateParam(req *v1.ProjectUpdateRequest) error {
	if req.Id < 1{
		return errors.ParamsError
	}
	if req.Name == "" {
		return status.Error(10003, "项目名称不能为空")
	}
	if req.ProjectType == "" {
		return status.Error(10003, "项目类型不能为空")
	}
	if req.CoveredArea == 0 {
		return status.Error(10003, "建筑面积不能为空")
	}
	if req.StartDay == 0 {
		return status.Error(10003, "计划开工日期不能为空")
	}
	if req.EndDay == 0 {
		return status.Error(10003, "计划竣工日期不能为空")
	}
	if req.EndDay <= req.StartDay {
		return status.Error(10003, "计划竣工日期不能早于计划开工日期")
	}
	if req.ProjectFunction == "" {
		return status.Error(10003, "项目功能不能为空")
	}
	if req.TotalLevel == 0 {
		return status.Error(10003, "总层数不能为空")
	}
	if req.Phone == "" {
		return status.Error(10003, "电话不能为空")
	}
	if req.ProjectLeader == "" {
		return status.Error(10003, "项目负责人不能为空")
	}
	if req.SafetyRecordNo == "" {
		return status.Error(10003, "安监备案号不能为空")
	}
	if req.Construction == "" {
		return status.Error(10003, "施工单位不能为空")
	}
	if req.Development == "" {
		return status.Error(10003, "建筑单位不能为空")
	}
	if req.Oversee == "" {
		return status.Error(10003, "监理单位不能为空")
	}
	if req.Design == "" {
		return status.Error(10003, "设计单位不能为空")
	}
	if req.Exploration == "" {
		return status.Error(10003, "勘察单位不能为空")
	}
	if req.Lon == 0 {
		return status.Error(10003, "经度不能为空")
	}
	if req.Lat == 0 {
		return status.Error(10003, "纬度不能为空")
	}
	if req.Location == "" {
		return status.Error(10003, "项目所在地不能为空")
	}

	if !utils.VerifyMobileFormat(req.Phone) {
		return status.Error(10003, "业务负责人手机号格式错误")
	}
	return nil
}

func ProjectUpdate(ctx context.Context, req *v1.ProjectUpdateRequest) (reply *v1.ProjectUpdateReply, err error) {
	if err := checkProjectUpdateParam(req); err != nil {
		return nil, err
	}
	now := time.Now()
	reply = &v1.ProjectUpdateReply{}
	updateTask := func(db *gorm.DB) error {
		p := model.TProject{}
		and := map[string]interface{}{
			"id !=":req.Id,
			//"safety_record_no":          req.SafetyRecordNo,
		}
		or := map[string]interface{}{
			"safety_record_no":          req.SafetyRecordNo,
			"name":req.Name,
		}
		err = p.FindOr(database.DB(), and, or)
		if err != nil && err != gorm.ErrRecordNotFound{
			logger.Error("ProjectUpdate",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if p.Id > 0 {
			return errors.ProjectExistError
		}
		p = model.TProject{}
		and = map[string]interface{}{
			"id":req.Id,
		}
		err = p.FindOr(database.DB(), and, nil)
		if err != nil && err != gorm.ErrRecordNotFound{
			logger.Error("ProjectUpdate",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if p.Id == 0 {
			return errors.ProjectNotExistError
		}
		m := map[string]interface{}{}
		bytes, _ := json.Marshal(req)
		json.Unmarshal(bytes, &m)
		delete(m, "cid")
		delete(m, "id")
		delete(m, "project_no")
		delete(m, "is_finish")
		delete(m, "project_plan")
		delete(m, "project_effect_pic")
		m["project_effect_pic"] = ""
		m["project_plan"] = ""
		if len(req.ProjectEffectPic) > 0 {
			bytes, _ := json.Marshal(req.ProjectEffectPic)
			m["project_effect_pic"] = string(bytes)
		}
		if len(req.ProjectPlan) > 0 {
			bytes, _ := json.Marshal(req.ProjectPlan)
			m["project_plan"] = string(bytes)
		}
		m["updated_at"] = now
		m["start_day"] = time.Unix(req.StartDay, 0)
		m["end_day"] = time.Unix(req.EndDay, 0)
		p = model.TProject{}
		and = map[string]interface{}{
			"id":req.Id,
		}
		err := p.Update(database.DB(), and, m)
		if err != nil {
			logger.Error("ProjectUpdate",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		return nil
	}


	err = transtasker.Exec(database.DB(),
		transtasker.NewTask(updateTask))

	return reply, err


}
